@@ -26,6 +26,8 @@ Dependents <- setRefClass(
 )
 
 
+# ReactiveValues ------------------------------------------------------------
+
 ReactiveValues <- setRefClass(
   'ReactiveValues',
   fields = list(
@@ -112,13 +114,14 @@ ReactiveValues <- setRefClass(
 )
 
 
-# reactivevalues: S3 wrapper class for Values class -----------------------
+# reactivevalues ------------------------------------------------------------
+# S3 wrapper class for ReactiveValues reference class
 
 #' Create an object for storing reactive values
 #'
 #' This function returns an object for storing reactive values. It is similar
 #' to a list, but with special capabilities for reactive programming. When you
-#' read a value from it, the calling reactive function takes a reactive
+#' read a value from it, the calling reactive expression takes a reactive
 #' dependency on that value, and when you write to it, it notifies any reactive
 #' functions that depend on that value.
 #'
@@ -168,10 +171,7 @@ reactiveValues <- function(...) {
 # @param values A ReactiveValues object
 # @param readonly Should this object be read-only?
 .createReactiveValues <- function(values = NULL, readonly = FALSE) {
-  acc <- list(impl=values)
-  class(acc) <- 'reactivevalues'
-  attr(acc, 'readonly') <- readonly
-  return(acc)
+  structure(list(impl=values), class='reactivevalues', readonly=readonly)
 }
 
 #' @S3method $ reactivevalues
@@ -219,7 +219,7 @@ names.reactivevalues <- function(x) {
 
 #' @S3method as.list reactivevalues
 as.list.reactivevalues <- function(x, all.names=FALSE, ...) {
-  .Deprecated("reactiveValuesToList",
+  shinyDeprecated("reactiveValuesToList",
     msg = paste("'as.list.reactivevalues' is deprecated. ",
       "Use reactiveValuesToList instead.",
       "\nPlease see ?reactiveValuesToList for more information.",
@@ -254,25 +254,28 @@ reactiveValuesToList <- function(x, all.names=FALSE) {
   .subset2(x, 'impl')$toList(all.names)
 }
 
+# Observable ----------------------------------------------------------------
+
 Observable <- setRefClass(
   'Observable',
   fields = list(
     .func = 'function',
     .label = 'character',
     .dependents = 'Dependents',
-    .dirty = 'logical',
+    .invalidated = 'logical',
     .running = 'logical',
     .value = 'ANY',
+    .visible = 'logical',
     .execCount = 'integer'
   ),
   methods = list(
     initialize = function(func, label=deparse(substitute(func))) {
       if (length(formals(func)) > 0)
-        stop("Can't make a reactive function from a function that takes one ",
+        stop("Can't make a reactive expression from a function that takes one ",
              "or more parameters; only functions without parameters can be ",
              "reactive.")
       .func <<- func
-      .dirty <<- TRUE
+      .invalidated <<- TRUE
       .running <<- FALSE
       .label <<- label
       .execCount <<- 0L
@@ -280,70 +283,95 @@ Observable <- setRefClass(
     getValue = function() {
       .dependents$register()
 
-      if (.dirty || .running) {
+      if (.invalidated || .running) {
         .self$.updateValue()
       }
       
       if (identical(class(.value), 'try-error'))
         stop(attr(.value, 'condition'))
-      return(.value)
+
+      if (.visible)
+        .value
+      else
+        invisible(.value)
     },
     .updateValue = function() {
       ctx <- Context$new(.label)
       ctx$onInvalidate(function() {
-        .dirty <<- TRUE
+        .invalidated <<- TRUE
         .dependents$invalidate()
       })
       .execCount <<- .execCount + 1L
 
-      .dirty <<- FALSE
+      .invalidated <<- FALSE
 
       wasRunning <- .running
       .running <<- TRUE
       on.exit(.running <<- wasRunning)
 
       ctx$run(function() {
-        .value <<- try(.func(), silent=FALSE)
+        result <- withVisible(try(.func(), silent=FALSE))
+        .visible <<- result$visible
+        .value <<- result$value
       })
     }
   )
 )
 
-#' Create a Reactive Function
-#' 
-#' Wraps a normal function to create a reactive function. Conceptually, a 
-#' reactive function is a function whose result will change over time.
-#' 
-#' Reactive functions are functions that can read reactive values and call other
-#' reactive functions. Whenever a reactive value changes, any reactive functions
-#' that depended on it are marked as "invalidated" and will automatically 
-#' re-execute if necessary. If a reactive function is marked as invalidated, any
-#' other reactive functions that recently called it are also marked as 
-#' invalidated. In this way, invalidations ripple through the functions that 
+#' Create a reactive expression
+#'
+#' Wraps a normal expression to create a reactive expression. Conceptually, a
+#' reactive expression is a expression whose result will change over time.
+#'
+#' Reactive expressions are expressions that can read reactive values and call other
+#' reactive expressions. Whenever a reactive value changes, any reactive expressions
+#' that depended on it are marked as "invalidated" and will automatically
+#' re-execute if necessary. If a reactive expression is marked as invalidated, any
+#' other reactive expressions that recently called it are also marked as
+#' invalidated. In this way, invalidations ripple through the expressions that
 #' depend on each other.
-#' 
-#' See the \href{http://rstudio.github.com/shiny/tutorial/}{Shiny tutorial} for 
-#' more information about reactive functions.
-#' 
-#' @param x The value or function to make reactive. The function must not have 
-#'   any parameters.
-#' @return A reactive function. (Note that reactive functions can only be called
-#'   from within other reactive functions.)
-#'   
+#'
+#' See the \href{http://rstudio.github.com/shiny/tutorial/}{Shiny tutorial} for
+#' more information about reactive expressions.
+#'
+#' @param x An expression (quoted or unquoted).
+#' @param env The parent environment for the reactive expression. By default, this
+#'   is the calling environment, the same as when defining an ordinary
+#'   non-reactive expression.
+#' @param quoted Is the expression quoted? By default, this is \code{FALSE}.
+#'   This is useful when you want to use an expression that is stored in a
+#'   variable; to do so, it must be quoted with `quote()`.
+#' @param label A label for the reactive expression, useful for debugging.
+#'
+#' @examples
+#' values <- reactiveValues(A=1)
+#'
+#' reactiveB <- reactive({
+#'   values$A + 1
+#' })
+#'
+#' # Can use quoted expressions
+#' reactiveC <- reactive(quote({ values$A + 2 }), quoted = TRUE)
+#'
+#' # To store expressions for later conversion to reactive, use quote()
+#' expr_q <- quote({ values$A + 3 })
+#' reactiveD <- reactive(expr_q, quoted = TRUE)
+#'
+#' # View the values from the R console with isolate()
+#' isolate(reactiveB())
+#' isolate(reactiveC())
+#' isolate(reactiveD())
+#'
 #' @export
-reactive <- function(x) {
-  UseMethod("reactive")
-}
-#' @S3method reactive function
-reactive.function <- function(x) {
-  return(Observable$new(x, deparse(substitute(x)))$getValue)
-}
-#' @S3method reactive default
-reactive.default <- function(x) {
-  stop("Don't know how to make this object reactive!")
+reactive <- function(x, env = parent.frame(), quoted = FALSE, label = NULL) {
+  fun <- exprToFunction(x, env, quoted)
+  if (is.null(label))
+    label <- deparse(body(fun))
+
+  Observable$new(fun, label)$getValue
 }
 
-# Return the number of times that a reactive function or observer has been run
+# Return the number of times that a reactive expression or observer has been run
 execCount <- function(x) {
   if (is.function(x))
     return(environment(x)$.execCount)
@@ -353,16 +381,20 @@ execCount <- function(x) {
     stop('Unexpected argument to execCount')
 }
 
+# Observer ------------------------------------------------------------------
+
 Observer <- setRefClass(
   'Observer',
   fields = list(
     .func = 'function',
     .label = 'character',
-    .flushCallbacks = 'list',
-    .execCount = 'integer'
+    .invalidateCallbacks = 'list',
+    .execCount = 'integer',
+    .onResume = 'function',
+    .suspended = 'logical'
   ),
   methods = list(
-    initialize = function(func, label) {
+    initialize = function(func, label, suspended = FALSE) {
       if (length(formals(func)) > 0)
         stop("Can't make an observer from a function that takes parameters; ",
              "only functions without parameters can be reactive.")
@@ -370,68 +402,133 @@ Observer <- setRefClass(
       .func <<- func
       .label <<- label
       .execCount <<- 0L
+      .suspended <<- suspended
+      .onResume <<- function() NULL
 
       # Defer the first running of this until flushReact is called
-      ctx <- Context$new(.label)
-      ctx$onFlush(function() {
-        run()
-      })
-      ctx$addPendingFlush()
+      .createContext()$invalidate()
     },
-    run = function() {
+    .createContext = function() {
       ctx <- Context$new(.label)
 
       ctx$onInvalidate(function() {
-        lapply(.flushCallbacks, function(func) {
+        lapply(.invalidateCallbacks, function(func) {
           func()
           NULL
         })
-        ctx$addPendingFlush()
-      })
 
+        continue <- function() {
+          ctx$addPendingFlush()
+        }
+        
+        if (.suspended == FALSE)
+          continue()
+        else
+          .onResume <<- continue
+      })
+      
       ctx$onFlush(function() {
         run()
       })
+      
+      return(ctx)
+    },
+    run = function() {
+      ctx <- .createContext()
       .execCount <<- .execCount + 1L
       ctx$run(.func)
     },
     onInvalidate = function(func) {
-      .flushCallbacks <<- c(.flushCallbacks, func)
+      "Register a function to run when this observer is invalidated"
+      .invalidateCallbacks <<- c(.invalidateCallbacks, func)
+    },
+    suspend = function() {
+      "Causes this observer to stop scheduling flushes (re-executions) in
+      response to invalidations. If the observer was invalidated prior to this
+      call but it has not re-executed yet (because it waits until onFlush is
+      called) then that re-execution will still occur, becasue the flush is
+      already scheduled."
+      .suspended <<- TRUE
+    },
+    resume = function() {
+      "Causes this observer to start re-executing in response to invalidations.
+      If the observer was invalidated while suspended, then it will schedule
+      itself for re-execution (pending flush)."
+      if (.suspended) {
+        .suspended <<- FALSE
+        .onResume()
+        .onResume <<- function() NULL
+      }
+      invisible()
     }
   )
 )
 
 #' Create a reactive observer
 #' 
-#' Creates an observer from the given function. An observer is like a reactive 
-#' function in that it can read reactive values and call reactive functions, and
+#' Creates an observer from the given expression An observer is like a reactive
+#' expression in that it can read reactive values and call reactive expressions, and
 #' will automatically re-execute when those dependencies change. But unlike 
-#' reactive functions, it doesn't yield a result and can't be used as an input 
-#' to other reactive functions. Thus, observers are only useful for their side 
+#' reactive expression, it doesn't yield a result and can't be used as an input 
+#' to other reactive expressions. Thus, observers are only useful for their side 
 #' effects (for example, performing I/O).
 #' 
-#' Another contrast between reactive functions and observers is their execution
-#' strategy. Reactive functions use lazy evaluation; that is, when their
+#' Another contrast between reactive expressions and observers is their execution
+#' strategy. Reactive expressions use lazy evaluation; that is, when their
 #' dependencies change, they don't re-execute right away but rather wait until
 #' they are called by someone else. Indeed, if they are not called then they
 #' will never re-execute. In contrast, observers use eager evaluation; as soon
 #' as their dependencies change, they schedule themselves to re-execute.
 #' 
-#' @param func The function to observe. It must not have any parameters. Any 
-#'   return value from this function will be ignored.
-#'   
+#' @param x An expression (quoted or unquoted). Any return value will be ignored.
+#' @param env The parent environment for the reactive expression. By default, this
+#'   is the calling environment, the same as when defining an ordinary
+#'   non-reactive expression.
+#' @param quoted Is the expression quoted? By default, this is \code{FALSE}.
+#'   This is useful when you want to use an expression that is stored in a
+#'   variable; to do so, it must be quoted with `quote()`.
+#' @param label A label for the observer, useful for debugging.
+#' @param suspended If \code{TRUE}, start the observer in a suspended state.
+#'   If \code{FALSE} (the default), start in a non-suspended state.
+#'
+#' @examples
+#' values <- reactiveValues(A=1)
+#'
+#' obsB <- observe({
+#'   print(values$A + 1)
+#' })
+#'
+#' # Can use quoted expressions
+#' obsC <- observe(quote({ print(values$A + 2) }), quoted = TRUE)
+#'
+#' # To store expressions for later conversion to observe, use quote()
+#' expr_q <- quote({ print(values$A + 3) })
+#' obsD <- observe(expr_q, quoted = TRUE)
+#'
+#' # In a normal Shiny app, the web client will trigger flush events. If you
+#' # are at the console, you can force a flush with flushReact()
+#' shiny:::flushReact()
+#'
 #' @export
-observe <- function(func) {
-  invisible(Observer$new(func, deparse(substitute(func))))
+observe <- function(x, env=parent.frame(), quoted=FALSE, label=NULL,
+                    suspended=FALSE) {
+
+  fun <- exprToFunction(x, env, quoted)
+  if (is.null(label))
+    label <- deparse(body(fun))
+
+  invisible(Observer$new(fun, label=label, suspended=suspended))
 }
 
+# ---------------------------------------------------------------------------
+
 #' Timer
 #' 
 #' Creates a reactive timer with the given interval. A reactive timer is like a 
 #' reactive value, except reactive values are triggered when they are set, while
 #' reactive timers are triggered simply by the passage of time.
 #' 
-#' \link[=reactive]{Reactive functions} and observers that want to be 
+#' \link[=reactive]{Reactive expressions} and observers that want to be 
 #' invalidated by the timer need to call the timer function that 
 #' \code{reactiveTimer} returns, even if the current time value is not actually 
 #' needed.
@@ -485,22 +582,32 @@ invalidateLater <- function(millis) {
 
 #' Create a non-reactive scope for an expression
 #' 
-#' Executes the given expression in a scope where reactive values or functions 
+#' Executes the given expression in a scope where reactive values or expression 
 #' can be read, but they cannot cause the reactive scope of the caller to be 
 #' re-evaluated when they change.
 #' 
 #' Ordinarily, the simple act of reading a reactive value causes a relationship 
 #' to be established between the caller and the reactive value, where a change 
 #' to the reactive value will cause the caller to re-execute. (The same applies 
-#' for the act of getting a reactive function's value.) The \code{isolate} 
-#' function lets you read a reactive value or function without establishing this
+#' for the act of getting a reactive expression's value.) The \code{isolate} 
+#' function lets you read a reactive value or expression without establishing this
 #' relationship.
 #' 
-#' @param expr An expression that can access reactive values or functions.
+#' The expression given to \code{isolate()} is evaluated in the calling
+#' environment. This means that if you assign a variable inside the
+#' \code{isolate()}, its value will be visible outside of the \code{isolate()}.
+#' If you want to avoid this, you can use \code{\link{local}()} inside the
+#' \code{isolate()}.
+#'
+#' This function can also be useful for calling reactive expression at the
+#' console, which can be useful for debugging. To do so, simply wrap the
+#' calls to the reactive expression with \code{isolate()}.
+#'
+#' @param expr An expression that can access reactive values or expressions.
 #' 
 #' @examples
 #' \dontrun{
-#' observer(function() {
+#' observe({
 #'   input$saveButton  # Do take a dependency on input$saveButton
 #'   
 #'   # isolate a simple expression
@@ -508,7 +615,7 @@ invalidateLater <- function(millis) {
 #'   writeToDatabase(data)
 #' })
 #' 
-#' observer(function() {
+#' observe({
 #'   input$saveButton  # Do take a dependency on input$saveButton
 #'   
 #'   # isolate a whole block
@@ -519,11 +626,34 @@ invalidateLater <- function(millis) {
 #'   })
 #'   writeToDatabase(data)
 #' })
+#'
+#' observe({
+#'   x <- 1
+#'   # x outside of isolate() is affected
+#'   isolate(x <- 2)
+#'   print(x) # 2
+#'
+#'   y <- 1
+#'   # Use local() to avoid affecting calling environment
+#'   isolate(local(y <- 2))
+#'   print(y) # 1
+#' })
+#'
 #' }
+#'
+#' # Can also use isolate to call reactive expressions from the R console
+#' values <- reactiveValues(A=1)
+#' fun <- reactive({ as.character(values$A) })
+#' isolate(fun())
+#' # "1"
+#'
+#' # isolate also works if the reactive expression accesses values from the
+#' # input object, like input$x
+#'
 #' @export
 isolate <- function(expr) {
   ctx <- Context$new('[isolate]')
   ctx$run(function() {
-    eval.parent(expr)
+    expr
   })
 }
