@@ -278,8 +278,13 @@
       this.lastSentValues[name] = jsonValue;
       this.target.setInput(name, value);
     };
-    this.reset = function() {
-      this.lastSentValues = {};
+    this.reset = function(values) {
+      values = values || {};
+      var strValues = {};
+      $.each(values, function(key, value) {
+        strValues[key] = JSON.stringify(value);
+      });
+      this.lastSentValues = strValues;
     };
   }).call(InputNoResendDecorator.prototype);
 
@@ -1007,17 +1012,22 @@
     },
     getValue: function(el) {
       var numberVal = $(el).val();
-      if (!isNaN(numberVal))
+      if (/^\s*$/.test(numberVal))  // Return null if all whitespace
+        return null;
+      else if (!isNaN(numberVal))   // If valid Javascript number string, coerce to number
         return +numberVal;
       else
-        return numberVal;
+        return numberVal;           // If other string like "1e6", send it unchanged
+    },
+    getType: function(el) {
+      return "number"
     }
   });
   inputBindings.register(numberInputBinding, 'shiny.numberInput');
 
 
   var sliderInputBinding = {};
-  $.extend(sliderInputBinding, numberInputBinding, {
+  $.extend(sliderInputBinding, textInputBinding, {
     find: function(scope) {
       // Check if jslider plugin is loaded
       if (!$.fn.slider)
@@ -1278,6 +1288,7 @@
 
       // Send later in case DOM layout isn't final yet.
       setTimeout(sendPlotSize, 0);
+      setTimeout(sendOutputHiddenState, 0);
     }
 
     function unbindOutputs(scope) {
@@ -1302,9 +1313,9 @@
         return $(el).val();
     }
 
-    var inputs = new InputNoResendDecorator(new InputBatchSender(shinyapp));
-    var inputsRate = new InputRateDecorator(inputs);
-    var inputsDefer = new InputDeferDecorator(inputs);
+    var inputsNoResend = new InputNoResendDecorator(new InputBatchSender(shinyapp));
+    var inputsRate = new InputRateDecorator(inputsNoResend);
+    var inputsDefer = new InputDeferDecorator(inputsNoResend);
 
     inputs = inputsRate;
     $('input[type="submit"], button[type="submit"]').each(function() {
@@ -1372,7 +1383,7 @@
           var ratePolicy = binding.getRatePolicy();
           if (ratePolicy != null) {
             inputsRate.setRatePolicy(
-              id,
+              effectiveId,
               ratePolicy.policy,
               ratePolicy.delay);
           }
@@ -1490,15 +1501,37 @@
     // The server needs to know the size of each plot output element, in case
     // the plot is auto-sizing
     $('.shiny-plot-output').each(function() {
-      var width = this.offsetWidth;
-      var height = this.offsetHeight;
-      initialValues['.shinyout_' + this.id + '_width'] = width;
-      initialValues['.shinyout_' + this.id + '_height'] = height;
+      if (this.offsetWidth !== 0 || this.offsetHeight !== 0) {
+        initialValues['.shinyout_' + this.id + '_width'] = this.offsetWidth;
+        initialValues['.shinyout_' + this.id + '_height'] = this.offsetHeight;
+      }
     });
     function sendPlotSize() {
       $('.shiny-plot-output').each(function() {
-        inputs.setInput('.shinyout_' + this.id + '_width', this.offsetWidth);
-        inputs.setInput('.shinyout_' + this.id + '_height', this.offsetHeight);
+        if (this.offsetWidth !== 0 || this.offsetHeight !== 0) {
+          inputs.setInput('.shinyout_' + this.id + '_width', this.offsetWidth);
+          inputs.setInput('.shinyout_' + this.id + '_height', this.offsetHeight);
+        }
+      });
+    }
+
+    // Set initial state of outputs to hidden, if needed
+    $('.shiny-bound-output').each(function() {
+      if (this.offsetWidth === 0 && this.offsetHeight === 0) {
+        initialValues['.shinyout_' + this.id + '_hidden'] = true;
+      } else {
+        initialValues['.shinyout_' + this.id + '_hidden'] = false;
+      }
+    });
+    // Send update when hidden state changes
+    function sendOutputHiddenState() {
+      $('.shiny-bound-output').each(function() {
+        // Assume that the object is hidden when width and height are 0
+        if (this.offsetWidth === 0 && this.offsetHeight === 0) {
+          inputs.setInput('.shinyout_' + this.id + '_hidden', true);
+        } else {
+          inputs.setInput('.shinyout_' + this.id + '_hidden', false);
+        }
       });
     }
     // The size of each plot may change either because the browser window was
@@ -1506,9 +1539,12 @@
     // of 0x0). It's OK to over-report sizes because the input pipeline will
     // filter out values that haven't changed.
     $(window).resize(debounce(500, sendPlotSize));
-    $('body').on('shown.sendPlotSize hidden.sendPlotSize', '*', sendPlotSize);
+    $('body').on('shown.sendPlotSize', '*', sendPlotSize);
+    $('body').on('shown.sendOutputHiddenState hidden.sendOutputHiddenState', '*',
+                 sendOutputHiddenState);
 
     // We've collected all the initial values--start the server process!
+    inputsNoResend.reset(initialValues);
     shinyapp.connect(initialValues);
   } // function initShiny()
 
